{% load admin_modify %}
<fieldset class="module">
   <h2>{{ bound_related_object.relation.opts.verbose_name_plural|capfirst|escape }}</h2><table>
   <thead><tr>
   {% for fw in bound_related_object.field_wrapper_list %}
      {% if fw.needs_header %}
         <th{{ fw.header_class_attribute }}>{{ fw.field.verbose_name|capfirst|escape }}</th>
      {% endif %}
   {% endfor %}
   </tr></thead>
   {% for fcw in bound_related_object.form_field_collection_wrappers %}
      {% if change %}{% if original_row_needed %}
         {% if fcw.obj.original %}
            <tr class="row-label {% cycle row1,row2 %}"><td colspan="{{ num_headers }}"><strong>{{ fcw.obj.original }}</strong></tr>
         {% endif %}
      {% endif %}{% endif %}
      {% if fcw.obj.errors %}
         <tr class="errorlist"><td colspan="{{ num_headers }}">
            {{ fcw.obj.html_combined_error_list }}
         </tr>
      {% endif %}
      <tr class="{% cycle row1,row2 %}">
      {% for bound_field in fcw.bound_fields %}
         {% if not bound_field.hidden %}
         <td {{ bound_field.cell_class_attribute }}>
            {% field_widget bound_field %}
         </td>
         {% endif %}
      {% endfor %}
      {% if bound_related_object.show_url %}<td>
         {% if fcw.obj.original %}<a href="/r/{{ fcw.obj.original.content_type_id }}/{{ fcw.obj.original.id }}/">View on site</a>{% endif %}
      </td>{% endif %}
      </tr>

   {% endfor %} </table>

   {% for fcw in bound_related_object.form_field_collection_wrappers %}
      {% for bound_field in fcw.bound_fields %}
         {% if bound_field.hidden %}
            {% field_widget bound_field %}
         {% endif %}
      {% endfor %}
   {% endfor %}
</fieldset>
