{% load i18n %}
{% trans "You're receiving this e-mail because you requested a password reset" %}
{% blocktrans %}for your user account at {{ site_name }}{% endblocktrans %}.

{% blocktrans %}Your new password is: {{ new_password }}{% endblocktrans %}

{% trans "Feel free to change this password by going to this page:" %}

http://{{ domain }}/password_change/

{% trans "Your username, in case you've forgotten:" %} {{ user.username }}

{% trans "Thanks for using our site!" %}

{% blocktrans %}The {{ site_name }} team{% endblocktrans %}
